#include "twn4.sys.h"
#include "apptools.h"

#define KEY_CNT		3
const byte Keys[KEY_CNT][6] =
{
	{ 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF },
	{ 0xA0,0xA1,0xA2,0xA3,0xA4,0xA5 },
	{ 0xD3,0xF7,0xD3,0xF7,0xD3,0xF7 }
};

#define MAXIDBYTES  8
#define MAXIDBITS   (MAXIDBYTES*8)


int main(void)
{
	byte ID[MAXIDBYTES];
	int IDBitCnt;
	int TagType;
	
    // Init LEDs
    LEDInit(REDLED | GREENLED);
    // Turn on green LED
    LEDOn(GREENLED);
    // Turn off red LED
    LEDOff(REDLED);
    // Make some noise at startup at low volume
    SetVolume(30);
    BeepLow();
    BeepHigh();
    // Continue with maximum  volume
    SetVolume(100);
    
    SetTagTypes(0,TAGMASK(HFTAG_MIFARE));

MainLoop:
    // Turn off the green LED
    LEDOff(GREENLED);
    // Let the red one blink, start with on-state
    LEDOn(REDLED);

	while (!SearchTag(&TagType,&IDBitCnt,ID,sizeof(ID)))
		;

    if (TagType != HFTAG_MIFARE)
        goto MainLoop;

    // We found a tag
    BeepHigh();

    // Print the ID
    WriteString("UID ");
    WriteHex(ID,IDBitCnt,(IDBitCnt+7)/8*2);
    WriteChar('\r');

	byte Sector;
    for (Sector=0; Sector<64; Sector += Sector < 0x20 ? 1 : 4)
    {
	    WriteString("Sector ");
	    WriteHex(&Sector,8,2);
	    WriteChar(' ');
	    
	    int KeyIndex;
	    for (KeyIndex=0; KeyIndex<KEY_CNT; KeyIndex++)
	    {
	        if (!SearchTag(&TagType,&IDBitCnt,ID,sizeof(ID)))
	        {
	            // Tag disappeared
			    WriteString("Tag removed\r");
			    goto MainLoop;
	        }
	        if (MifareClassic_Login(Keys[KeyIndex],KEYA,Sector))
	        	break;
	        if (MifareClassic_Login(Keys[KeyIndex],KEYB,Sector))
	        	break;
	    }
	    if (KeyIndex == KEY_CNT)
	    {
	        WriteString("Unknown key\r");
	        continue;
	    }

        // We logged into the current sector
		LEDOn(GREENLED);
		LEDOff(REDLED);

        WriteString("Key ");
        WriteHex(Keys[KeyIndex],48,12);
	    WriteChar('\r');

        // Dump the content of this sector
        int BlockCount = Sector < 0x20 ? 4 : 16;
        byte Block;
        for (Block=0; Block<BlockCount; Block++)
        {
            byte Address = Sector*4+Block;
            WriteHex(&Address,8,2);
            WriteChar(' ');
			byte Data[16];
            if (!MifareClassic_ReadBlock(Address,Data))
            {
                WriteChar('\r');
                break;
            }
            WriteHex(Data,128,32);
            WriteChar('\r');
        }
    }
	WriteString("End\r");
    while (SearchTag(&TagType,&IDBitCnt,ID,sizeof(ID)))
    	;
    goto MainLoop;
}
