// ******************************************************************
//
//    File: app_standard.c
//    Date: 2013-02-07
// Version: 2
//
// Purpose:
//
// This is the standard app for TWN4 readers, which is installed
// as default app on TWN4 readers. This script can be run on any
// type of TWN4 reader without modification.
// 
// Feel free to modify this program for your specific purposes!
//
// V1:
// ---
// - Initial release
//
// V2:
// ---
// - Support configuration via AppBlaster
//
// ******************************************************************

#include "twn4.sys.h"
#include "apptools.h"

#ifdef HAVECONFIG
    #include "app_config.h"
#else
    #define LFTAGTYPES          0xFFFFFFFF
    #define HFTAGTYPES          0xFFFFFFFF
    #define BIT_LENGTHS         0xFFFFFFFFFFFFFFFF
    #define SWAP_BITS           0
    #define SWAP_BYTES          0
    #define OUTPUT_ALL_BITS     1
    #define BITOFFSET           0
    #define BITCOUNT            0
    #define OUTPUT_RADIX        16
    #define OUTPUT_DIGITS       0
#endif

#define MAXIDBYTES  8
#define MAXIDBITS   (MAXIDBYTES*8)

byte ID[MAXIDBYTES];
int IDBitCnt;
int TagType;

byte LastID[MAXIDBYTES];
int LastIDBitCnt;
int LastTagType;

int main(void)
{
    // Show the startup message
    if (GetHostChannel() == CHANNEL_COM1)
    {
        // A V24 device is writing the version at startup
        WriteVersion();
        WriteChar('\r');
    }

#ifdef HAVECONFIG
    SetTagTypes(LFTAGTYPES,HFTAGTYPES);
#endif
    
    // Init LEDs
    LEDInit(REDLED | GREENLED);
    // Turn on green LED
    LEDOn(GREENLED);
    // Turn off red LED
    LEDOff(REDLED);
    // Make some noise at startup at low volume
    SetVolume(30);
    BeepLow();
    BeepHigh();
    // Continue with maximum  volume
    SetVolume(100);
    
    // No transponder found up to now
    LastTagType = NOTAG;
    while (true)
    {
        // Search a transponder
        if (SearchTag(&TagType,&IDBitCnt,ID,sizeof(ID)))
        {
            unsigned long long BitMask = 1ULL << (IDBitCnt-1);
            if (BitMask & BIT_LENGTHS)
            {
                // Is this transponder new to us?
                if (TagType != LastTagType || IDBitCnt != LastIDBitCnt || !CompBits(ID,0,LastID,0,MAXIDBITS))
                {
                    // Save this as known ID, before modifying the ID for proper output format
                    CopyBits(LastID,0,ID,0,MAXIDBITS);
                    LastIDBitCnt = IDBitCnt;
                    LastTagType = TagType;
                    
                    // Yes! Sound a beep
                    BeepHigh();
                    // Turn off the green LED
                    LEDOff(GREENLED);
                    // Let the red one blink, start with on-state
                    LEDOn(REDLED);
                    LEDBlink(REDLED,500,500);
                
#if SWAP_BITS
                    SwapBits(ID,0,IDBitCnt);
#endif                
#if SWAP_BYTES
                    SwapBytes(ID,(IDBitCnt+7)/8);
#endif
#if !OUTPUT_ALL_BITS
                    CopyBits(ID,0,ID,BITOFFSET,BITCOUNT);
                    IDBitCnt = BITCOUNT;
#endif

#if OUTPUT_DIGITS == 0 && OUTPUT_RADIX == 2

                    // Make number of digits identical with number of digits
                    WriteRadix(ID,IDBitCnt,IDBitCnt,OUTPUT_RADIX);

#elif OUTPUT_DIGITS == 0 && OUTPUT_RADIX == 16

                    // Make number of digits always a multiple of two (= one byte),
                    // where all bits fit into
                    WriteRadix(ID,IDBitCnt,(IDBitCnt+7)/8*2,OUTPUT_RADIX);
                    
#else

                    WriteRadix(ID,IDBitCnt,OUTPUT_DIGITS,OUTPUT_RADIX);
                    
#endif
                    WriteChar('\r');
                }
                // Start a timeout of two seconds
                StartTimer(2000);
            }
        }
        if (TestTimer())
        {
            LEDOn(GREENLED);
            LEDOff(REDLED);
            LastTagType = NOTAG;
        }
    }
}
