#include "twn4.sys.h"
#include "apptools.h"

#define MAXIDBYTES  8
#define MAXIDBITS   (MAXIDBYTES*8)

byte ID[MAXIDBYTES];
int IDBitCnt;
int TagType;

byte LastID[MAXIDBYTES];
int LastIDBitCnt;
int LastTagType;

const char *GetTagName(int TagType)
{
	switch (TagType)
	{
	case LFTAG_EM4102:    return "EM4102 / Casi Rusco";
	case LFTAG_HITAG1S:   return "Hitag 1 / Hitag S";
	case LFTAG_HITAG2:    return "Hitag 2";
	case LFTAG_EM4150:    return "EM4150";
	case LFTAG_AT5555:    return "AT55xx";
	case LFTAG_ISOFDX:    return "ISO FDX";
	case LFTAG_EM4026:    return "EM4026";
	case LFTAG_HITAGU:    return "Hitag U";
	case LFTAG_EM4305:    return "EM4305";
	case LFTAG_HIDPROX:   return "HID Prox";
	case LFTAG_TIRIS:     return "ISO HDX / Tiris";
	case LFTAG_COTAG:     return "Cotag";
	case LFTAG_IOPROX:    return "IOProx";
	case LFTAG_INDITAG:   return "Indala";
	case HFTAG_MIFARE:    return "ISO14443A / Mifare";
	case HFTAG_ISO14443B: return "ISO14443B";
	case HFTAG_ISO15693:  return "ISO15693";
	case HFTAG_LEGIC:     return "Legic Prime";
	case HFTAG_HIDICLASS: return "HID iClass";
	case HFTAG_FELICA:    return "Felica";
	case HFTAG_SRX:       return "SRX";
	case HFTAG_NFCP2P:    return "NFC";
	}
	return "Unknown";
}

int main(void)
{
	// Show the startup message
    if (GetHostChannel() == CHANNEL_COM1)
    {
        // A V24 device is writing the version at startup
        WriteVersion();
        WriteChar('\r');
    }
    SetTagTypes(0xFFFFFFFF,0xFFFFFFFF);
    
    // Init LEDs
    LEDInit(REDLED | GREENLED);
    // Turn on green LED
    LEDOn(GREENLED);
    // Turn off red LED
    LEDOff(REDLED);
    // Make some noise at startup at low volume
    SetVolume(30);
    BeepLow();
    BeepHigh();
    // Continue with maximum  volume
    SetVolume(100);
    
    // No transponder found up to now
    LastTagType = NOTAG;
    while (true)
    {
        // Search a transponder
        if (SearchTag(&TagType,&IDBitCnt,ID,sizeof(ID)))
        {
            // Is this transponder new to us?
            if (TagType != LastTagType || IDBitCnt != LastIDBitCnt || !CompBits(ID,0,LastID,0,MAXIDBITS))
            {
                // Save this as known ID, before modifying the ID for proper output format
                CopyBits(LastID,0,ID,0,MAXIDBITS);
                LastIDBitCnt = IDBitCnt;
                LastTagType = TagType;
                
                // Yes! Sound a beep
                BeepHigh();
                // Turn off the green LED
                LEDOff(GREENLED);
                // Let the red one blink, start with on-state
                LEDOn(REDLED);
                LEDBlink(REDLED,500,500);
                
                WriteString(GetTagName(TagType));
                WriteString("  ");

				WriteDec((byte*)&IDBitCnt,8,1);
                WriteString(" Bits  ID ");

                // Write ID with appropriate number of digits
                WriteHex(ID,IDBitCnt,(IDBitCnt+7)/8*2);
		        WriteChar('\r');
            }
            // Start a timeout of two seconds
            StartTimer(2000);
        }
        if (TestTimer())
        {
            LEDOn(GREENLED);
            LEDOff(REDLED);
            LastTagType = NOTAG;
        }
    }
}
