#include "twn4.sys.h"
#include "apptools.h"

// ******************************************************************
// ****** Timer Functions *******************************************
// ******************************************************************

static unsigned long TimerDuration;
static unsigned long TimerStartTime;
void StartTimer(unsigned long Duration)
{
    TimerDuration = Duration;
    TimerStartTime = GetSysTicks();
}
void StopTimer(void)
{
    TimerDuration = 0;
}
bool TestTimer(void)
{
    if (TimerDuration == 0)
        return false;
    if (GetSysTicks()-TimerStartTime < TimerDuration)
        return false;
    TimerDuration = 0;
    return true;
}

// ******************************************************************
// ****** Host Communication ****************************************
// ******************************************************************

void WriteChar(char Char)
{
    HostWriteByte(Char);
}
void WriteString(const char *String)
{
    while (*String)
        WriteChar(*String++);
}
void WriteRadix(const byte *ID,int BitCnt,int DigitCnt,int Radix)
{
    char String[128+1];
    ConvertBinaryToString(ID,0,BitCnt,String,Radix,DigitCnt,sizeof(String)-1);
    WriteString(String);
}
void WriteBin(const byte *ID,int BitCnt,int DigitCnt)
{
    WriteRadix(ID,BitCnt,DigitCnt,2);
}
void WriteDec(const byte *ID,int BitCnt,int DigitCnt)
{
    WriteRadix(ID,BitCnt,DigitCnt,10);
}
void WriteHex(const byte *ID,int BitCnt,int DigitCnt)
{
    WriteRadix(ID,BitCnt,DigitCnt,16);
}
void WriteVersion(void)
{
    char Version[30+1];
    GetVersionString(Version,sizeof(Version)-1);
    WriteString(Version);
}

// ******************************************************************
// ****** Beep Functions ********************************************
// ******************************************************************

static int Volume = 0;
void SetVolume(int NewVolume)
{
    Volume = NewVolume;
}
int GetVolume(void)
{
    return Volume;
}
void BeepLow(void)
{
    Beep(Volume,BEEP_FREQUENCY_LOW,50,50);
}
void BeepHigh(void)
{
    Beep(Volume,BEEP_FREQUENCY_HIGH,50,50);
}

// ******************************************************************
// ****** Compatibility to TWN3 *************************************
// ******************************************************************

int ConvertTagTypeToTWN3(int TagTypeTWN4)
{
    switch (TagTypeTWN4)
    {
    case NOTAG:           return 0;     // const byte TAGTYPE_NONE = 0;
    case HFTAG_MIFARE:    return 1;     // const byte TAGTYPE_MIFARE = 1;
    case LFTAG_HIDPROX:   return 2;     // const byte TAGTYPE_HIDPROX = 2;
    case HFTAG_LEGIC:     return 3;     // const byte TAGTYPE_LEGIC = 3;
    case LFTAG_EM4102:    return 4;     // const byte TAGTYPE_EM4102 = 4;
    case LFTAG_HITAG1S:   return 5;     // const byte TAGTYPE_HITAG1S = 5;
    case LFTAG_HITAG2:    return 6;     // const byte TAGTYPE_HITAG2 = 6;
    case LFTAG_EM4150:    return 7;     // const byte TAGTYPE_EM4150 = 7;
    case LFTAG_AT5555:    return 8;     // const byte TAGTYPE_AT5555 = 8;
    case LFTAG_ISOFDX:    return 9;     // const byte TAGTYPE_ISOFDX = 9;
    case LFTAG_EM4026:    return 10;    // const byte TAGTYPE_EM4026 = 10;
    case LFTAG_HITAGU:    return 11;    // Not defined for TWN3
    case LFTAG_EM4305:    return 12;    // Not defined for TWN3
    case LFTAG_TIRIS:     return 13;    // Not defined for TWN3
    case LFTAG_COTAG:     return 14;    // Not defined for TWN3
    case LFTAG_IOPROX:    return 15;    // Not defined for TWN3
    case HFTAG_HIDICLASS: return 20;    // const byte TAGTYPE_HIDICLASS = 20;
    case HFTAG_ISO15693:  return 21;    // const byte TAGTYPE_ISO15693 = 21;
    case LFTAG_INDITAG:   return 22;    // const byte TAGTYPE_INDITAG = 22;
    case HFTAG_ISO14443B: return 23;    // const byte TAGTYPE_ISO14443B = 23;
    case HFTAG_FELICA:    return 24;    // const byte TAGTYPE_FELICA = 24;
    case HFTAG_SRX:       return 25;    // const byte TAGTYPE_SRX = 25;
    case HFTAG_NFCP2P:    return 26;    // Not defined for TWN3
    }
    return 255;
}
